#ifndef CMWCAMLIGHT_H
#define CMWCAMLIGHT_H

#include "cmwidget.h"

namespace Ui {
class CMWCamLight;
}

class CMWCamLight : public CMWidget
{
    Q_OBJECT

public:
    explicit CMWCamLight(QWidget *parent ,int iCamID,bool bSetDefaultProperty);
    ~CMWCamLight();
    void SetLightData(int dlight,int slight);
    void GetLightData(int &dlight,int &slight);
    void SetLight(int dlight,int slight);
    void SetCamInfo(int exposure, int gain);
    void GetCamInfo(int &exposure, int &gain);
	void SetCamData(int exposure, int gain);
    int m_iCamID;
    bool m_bSetDefaultProperty;

    void MTimeFresh();
    int m_iCurLevel;

    void SetDefaultLight();
	void SetCamDefault();

	virtual void MFresh();

private:
    void SetDlight(int ilight);
    void SetSlight(int ilight);


private slots:

    void on_horizontalScrollBar_DLight_MouseRelease();

    void on_horizontalScrollBar_SLight_MouseRelease( );


    void on_pushButton_SaveAsDefault_clicked();

    void on_pushButton_RestoreDefault_clicked();

    void on_scrollBar_Exposure_MouseRelease(int);
    void on_scrollBar_Gain_MouseRelease(int);



    void on_btnHightLight_clicked();

private:
    Ui::CMWCamLight *ui;
};

#endif // CMWCAMLIGHT_H
